@@ -52,6 +52,11 @@ def fromarray(array):
         return array.type
 
 def fromnumpy(shape, dtype, masked=False):
+    if not isinstance(shape, tuple):
+        shape = (shape,)
+    if not isinstance(dtype, awkward.util.numpy.dtype):
+        dtype = awkward.util.numpy.dtype(dtype)
+
     if masked:
         return OptionType(fromnumpy(shape, dtype))
     elif dtype.subdtype is not None:
@@ -309,13 +314,6 @@ class ArrayType(Type):
         else:
             return self._to.dtype
 
-    @property
-    def jshape(self):
-        if isinstance(self._to, awkward.util.numpy.dtype):
-            return (self._takes, self._to)
-        else:
-            return (self._takes,) + self._to.jshape
-
     def _isnumpy(self, seen):
         if id(self) in seen:
             return False
@@ -389,10 +387,6 @@ class TableType(Type):
                 out.append((n, x.dtype))
         return awkward.util.numpy.dtype(out)
 
-    @property
-    def jshape(self):
-        return (dict((n, x if isinstance(x, awkward.util.numpy.dtype) else x.jshape) for n, x in self._fields.items()),)
-
     def _isnumpy(self, seen):
         if id(self) in seen:
             return False
@@ -469,10 +463,6 @@ class UnionType(Type):
     def dtype(self):
         raise TypeError("Union has no Numpy dtype")
 
-    @property
-    def jshape(self):
-        return ([x.jshape for x in self._possibilities],)
-
     def _isnumpy(self, seen):
         return False
 
@@ -552,10 +542,6 @@ class OptionType(Type):
     def dtype(self):
         return self._type.dtype
 
-    @property
-    def jshape(self):
-        return ([self._type.jshape, None],)
-
     def _isnumpy(self, seen):
         if id(self) in seen:
             return False
